@@ -31,7 +31,7 @@ hostname = socket.gethostname()
 def make_base64(imgfile):
     with open(imgfile, 'rb') as img:
         mtype = mimetypes.guess_type(imgfile)[0]
-        imbytes = base64.encodestring(img.read())
+        imbytes = base64.encodebytes(img.read())
         data = imbytes.decode('utf-8').strip()
         return u"data:{mtype};base64,{data}".format(mtype=mtype, data=data)
 
